@@ -2,9 +2,9 @@
 #
 #	XS.pm : perl function definition for Term::ReadLine::Gnu
 #
-#	$Id: XS.pm,v 1.7 1999-04-11 00:17:57+09 hayashi Exp $
+#	$Id: XS.pm,v 1.10 2000-04-03 00:58:32+09 hayashi Exp $
 #
-#	Copyright (c) 1999 Hiroo Hayashi.  All rights reserved.
+#	Copyright (c) 2000 Hiroo Hayashi.  All rights reserved.
 #
 #	This program is free software; you can redistribute it and/or
 #	modify it under the same terms as Perl itself.
@@ -56,7 +56,11 @@ my @_matches;
 
 1;
 
-__END__
+# Uncomment the following line to enable AutoSplit.  If you are using
+# AutoLoader.pm distributed with Perl 5.004 or earlier, you must
+# update AutoLoader.pm due to its bug.
+
+#__END__
 
 
 #
@@ -230,12 +234,16 @@ sub get_history_event ( $$;$ ) {
 # This module calls termcap (or its compatible) library, which the GNU
 # Readline Library already uses, instead of Term::Caps.pm.
 
+# Some terminals do not support 'ue' (underline end).
+use vars qw(%term_no_ue);
+%term_no_ue = ( kterm => 1 );
 
 sub ornaments {
     return $rl_term_set unless @_;
     $rl_term_set = shift;
     $rl_term_set ||= ',,,';
-    $rl_term_set = 'us,me,,' if $rl_term_set eq '1';
+    $rl_term_set = $term_no_ue{$ENV{TERM}} ? 'us,me,,' : 'us,ue,,'
+	if $rl_term_set eq '1';
     my @ts = split /,/, $rl_term_set, 4;
     my @rl_term_set
 	= map {$_ ? tgetstr($_) || '' : ''} @ts;
@@ -385,7 +393,7 @@ sub list_completion_function ( $$ ) {
 #
 sub _trp_completion_function ( $$ ) {
     my($text, $state) = @_;
-    
+
     my $cf;
     return undef unless defined ($cf = $Attribs{completion_function});
     
@@ -395,8 +403,11 @@ sub _trp_completion_function ( $$ ) {
 	# the first call
 	$_i = 0;		# clear index
 	@_matches = &$cf($text,
-			$Attribs{line_buffer},
-			$Attribs{point} - length($text));
+			 $Attribs{line_buffer},
+			 $Attribs{point} - length($text));
+	# return here since $#_matches is 0 instead of -1 when
+	# @_matches = undef
+	return undef unless defined $_matches[0];
     }
     
     for (; $_i <= $#_matches; $_i++) {
@@ -404,3 +415,7 @@ sub _trp_completion_function ( $$ ) {
     }
     return undef;
 }
+
+1;
+
+__END__
